import { Component, OnInit, ViewChild, Inject } from '@angular/core';
import {HttpClient} from '@angular/common/http'
import { MatDialogRef, MatDialog, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { MatSnackBar, MatSnackBarConfig } from '@angular/material/snack-bar';
import { KeySiteImgs } from './keysiteimgs.component'
import { ActivatedRoute } from '@angular/router';
import { TabbarAndScoreService } from 'src/app/http-interceptors/tabbar-and-score.service';



@Component({
  selector: 'app-key-site',
  templateUrl: './key-site.component.html',
  styleUrls: ['./key-site.component.scss']
})
export class KeySiteComponent implements OnInit {

  constructor(private tabbarService: TabbarAndScoreService,public http: HttpClient,public dialog: MatDialog,public snackBar: MatSnackBar,private route:ActivatedRoute,) { }

  displayedColumns: string[] = ['mainname', 'position', 'construction', 'nature', 'danger', 'img'];

  ngOnInit(): void {
    this.getAllBuilding()
  }

  allCompanyPosition:any = []; //所有单位重点部位
  companyTips:any; //所有单位重点提示
  companyInput:any; //单位重点提示数据
  allBuilding:any = []; //所有建筑 + 建筑重点部位 + 建筑重点提示

  //获取单位重点部位
  getCompanyPostion () {
    let id = {companyId:this.route.snapshot.queryParams.id}
    this.http.get('/api/CompanyImportantLocations',{params:id}).subscribe(data=>{
      this.allCompanyPosition = data
    })
  }

  //获取单位重点提示
  getCompanyTips () {
    let id = {companyId:this.route.snapshot.queryParams.id}
    this.http.get('/api/CompanyImportantTips',{params:id}).subscribe((data:any)=>{
      if (data) {
        this.companyTips = data
        this.companyInput = data.details}
    })
  }

  //获取所有建筑
  getAllBuilding () {
    let id = {companyId:this.route.snapshot.queryParams.id}
    this.http.get('/api/Buildings',{params:id}).subscribe((data:any)=>{
      if (data.length) {
        this.allBuilding = data
        this.allBuilding.forEach(element => {
          element.position = [] //建筑重点部位
          element.tips = null //建筑重点提示
          element.companyInput = null })  //建筑重点提示数据
        this.getAllBuildingPositon()
        this.getAllBuildingTips()
      } else {
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('暂无建筑数据','确定',config);
      }
    })
  }

  //获取所有建筑重点部位
  getAllBuildingPositon () {
    this.allBuilding.forEach(element => {
      let id = {buildingId:element.id}
      this.http.get('/api/BuildingImportantLocations',{params:id}).subscribe(data=>{
        element.position = data
      })
    });
  }

  //获取所有建筑重点提示
  getAllBuildingTips () {
    this.allBuilding.forEach(element => {
      let id = {buildingId:element.id}
      this.http.get('/api/BuildingImportantTips',{params:id}).subscribe((data:any)=>{
        if (data) {
          element.tips = data
          element.companyInput = data.details}
      })
    });
  }

  //保存单位重点提示
  Preservation () {
    if (this.companyTips) { //编辑单位重点提示
      let data = {
        companyId: this.companyTips.companyId,
        id: this.companyTips.id,
        details: this.companyInput}
      this.http.put(`/api/CompanyImportantTips/${this.companyTips.id}`,data).subscribe(data=>{
        this.getCompanyTips()
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('数据修改成功','确定',config);
        this.tabbarService.sendMessage('changeScore');//通知服务改变分数
      })
    } else { //新增单位重点提示
      let data = {
        companyId: this.route.snapshot.queryParams.id,
        id : "",
        details: this.companyInput || ""}
      this.http.post('/api/CompanyImportantTips',data).subscribe(data=>{
        this.getCompanyTips()
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('数据修改成功','确定',config);
        this.tabbarService.sendMessage('changeScore');//通知服务改变分数
      })
    }
  }

  //保存建筑重点提示
  PreservationBuilding (e) {
    let companyId  = this.route.snapshot.queryParams.id
    if (e.tips) { //编辑建筑重点提示
      let data = {
        buildingId: e.id,
        id: e.tips.id,
        details: e.companyInput}
      this.http.put(`/api/BuildingImportantTips/${e.tips.id}`,data,{params:{
        companyId : companyId 
      }}).subscribe(data=>{
        this.toUpdate(e)
      })
    } else { //新增重点单位提示
      let data = {
        buildingId: e.id,
        details: e.companyInput || ""}
      this.http.post('/api/BuildingImportantTips',data,{params:{
        companyId : companyId 
      }}).subscribe(data=>{
        this.toUpdate(e)
      })
    }
  }

  //查看图片
  seeImg (e) {
    if (e.imageUrls.length != 0) {
      let data = e
      const dialogRef = this.dialog.open(KeySiteImgs, {//调用open方法打开对话框并且携带参数过去
      data});
      dialogRef.afterClosed().subscribe();
    } else {
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('暂无图片数据','确定',config);
    }
  }

  objectName:any //上传对象名
  keySiteItem:any 
  keySiteItemImgList:any //重点部位图片集合
  //上传素材图片
  selectFile(element,e){
    let imgFile = e.target.files[0] || null //上传的文件
    this.keySiteItemImgList = element.imageUrls ? element.imageUrls : []
    this.keySiteItem = element
    console.log(this.keySiteItem)
    this.startUploading(imgFile)
  }
  
  startUploading (imgFile) {
    let _this = this
    let file = imgFile || null //获取上传的文件
    let fileSize = file.size || null //上传文件的总大小
    let shardSize = 5 * 1024 * 1024 //5MB一个分片
    let companyId = sessionStorage.getItem("companyId")
    if (file && fileSize <= shardSize) { //上传文件<=5MB时
      let formData = new FormData()
      formData.append("file",file)
      this.http.post(`api/Objects/WebPlan2D/${companyId}`,formData).subscribe((data:any)=>{
        this.objectName = data.objectName
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('上传成功','确定',config)
        this.keySiteItemImgList.push("/api/Objects/WebPlan2D/" + this.objectName)
        this.keySiteItem.imageUrls = this.keySiteItemImgList
        this.http.put(`/api/BuildingImportantLocations/${this.keySiteItem.id}`,this.keySiteItem,{
          params:{
            companyId : companyId
          }
        }).subscribe((data)=>{
          // console.log(666,'修改成功')
          const config = new MatSnackBarConfig();
          config.verticalPosition = 'top';
          config.duration = 3000
          this.snackBar.open('修改成功','确定',config)
        })
      })
    } else if (file && fileSize > shardSize) { //上传文件>5MB时,分块上传
      let config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('上传图片文件不允许大于5mb','确定',config);
    }

  }

  //封装方法获取更新当前tab页重点提示
  toUpdate (e) {
    let id = {buildingId:e.id}
    this.http.get('/api/BuildingImportantTips',{params:id}).subscribe((data:any)=>{
      if (data) {
        this.allBuilding.forEach(element => {
          if (element.id === e.id) {
            element.tips = data
            element.companyInput = data.details}
        });
      } // if
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('数据修改成功','确定',config);
      this.tabbarService.sendMessage('changeScore');//通知服务改变分数
    })
  }



}
