import { HttpClient } from "@angular/common/http";
import { Component, OnInit, Inject } from "@angular/core";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { ObjectsService } from "@src/app/services/objects.service";
import { DatePipe } from "@angular/common";
import { MatSnackBar } from "@angular/material/snack-bar";
import Viewer from "viewerjs";
@Component({
  selector: "app-upload-drill",
  templateUrl: "./upload-drill.component.html",
  styleUrls: ["./upload-drill.component.scss"],
  providers: [DatePipe],
})
export class UploadDrillComponent implements OnInit {
  constructor(
    public dialogRef: MatDialogRef<UploadDrillComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private objectsSer: ObjectsService,
    private http: HttpClient,
    private datePipe: DatePipe,
    public snackBar: MatSnackBar
  ) {}

  isEditMode = true;

  ngOnInit(): void {
    console.log(this.data);
    this.isEditMode = this.data.isEditMode;
    this.getData();
  }

  isHttpPost;
  getData() {
    this.http
      .get("/api/DrillRecords", {
        params: {
          CompanyId: this.data.unitInfo.id,
        },
      })
      .subscribe((data: any) => {
        console.log("演练记录数据", data);
        if (data.totalCount === 0) {
          this.isHttpPost = true;
        } else {
          this.isHttpPost = false;
          let item = data.items[0];
          this.drillId = item.id;
          this.teamLeader = item.teamLeader;
          this.time = [item.startTime, item.endTime];
          this.surveyPhoto = JSON.parse(item.images);
          this.waterPhoto = JSON.parse(item.waterSource);
          this.routePhoto = JSON.parse(item.escapeRoute);
          this.fireControlPhoto = JSON.parse(item.fireControlRoom);
        }
      });
  }

  surveyPhoto = [
    // "../../../../assets/images/yiyuan.png",
    // "../../../../assets/images/homeHeader.png",
    // "../../../../assets/images/add.png",
  ]; //调研人员照片
  waterPhoto = []; //水源照片
  routePhoto = []; //疏散照片
  fireControlPhoto = []; //消控室照片

  //设置文件路径并上传
  postFilePath;
  isLoading = false;
  async upload(e, type) {
    this.isLoading = true;
    let file = e.target.files[0] || null; //获取上传的文件
    let fileSize = file.size || null; //上传文件的总大小
    let shardSize = 5 * 1024 * 1024; //5MB 超过5MB要分块上传
    this.postFilePath = "演练记录/" + this.data.unitInfo.id;
    if (fileSize >= shardSize) {
      await this.postFileByMul(file, type);
    } else {
      await this.postFile(file, type);
    }
    this.isLoading = false;
  }

  //直接上传
  async postFile(file: File, type) {
    await new Promise((resolve, reject) => {
      this.objectsSer.postFile(this.postFilePath, file).subscribe((data) => {
        let dataObj = data as any;
        let filePath: string = ObjectsService.baseUrl + dataObj.objectName;
        this.setFileUrl(type, filePath);
        resolve("success");
      });
    });
  }
  //分块上传
  async postFileByMul(file: File, type) {
    await new Promise((resolve, reject) => {
      this.objectsSer
        .postFile_MultipartUpload(this.postFilePath, file)
        .then((value) => {
          console.log("分块上传成功", value);
          let dataObj = value as any;
          this.setFileUrl(type, dataObj.filePath + dataObj.fileName);
          resolve("success");
        });
    });
  }

  setFileUrl(type, filePath) {
    switch (type) {
      case "调研":
        this.surveyPhoto.push(filePath);
        break;
      case "水源":
        this.waterPhoto.push(filePath);
        break;
      case "疏散通道":
        this.routePhoto.push(filePath);
        break;
      case "消控室":
        this.fireControlPhoto.push(filePath);
        break;
    }
  }
  deleteImg(arr, key) {
    if (confirm("确认删除此照片吗")) {
      arr.splice(key, 1);
    }
  }
  teamLeader;
  time;
  drillId;
  isLoadingSave = false;
  save() {
    if (!this.isEditMode) {
      this.dialogRef.close();
      return;
    }

    if (!this.teamLeader || !this.time) {
      this.snackBar.open("请输入带队人员和熟悉时间", "确定", {
        verticalPosition: "top",
        duration: 3000,
      });
      return;
    }
    this.isLoadingSave = true;
    let body: any = {
      companyId: this.data.unitInfo.id,
      teamLeader: this.teamLeader,
      startTime: this.datePipe.transform(this.time[0], "yyyy-MM-ddTHH:mm:ss"),
      endTime: this.datePipe.transform(this.time[1], "yyyy-MM-ddTHH:mm:ss"),
      images: JSON.stringify(this.surveyPhoto),
      waterSource: JSON.stringify(this.waterPhoto),
      escapeRoute: JSON.stringify(this.routePhoto),
      fireControlRoom: JSON.stringify(this.fireControlPhoto),
    };
    if (this.isHttpPost) {
      this.http.post("/api/DrillRecords", body).subscribe((data: any) => {
        this.drillId = data.id;
        this.isHttpPost = false;
        this.isLoadingSave = false;
        this.snackBar.open("保存成功", "确定", {
          verticalPosition: "top",
          duration: 3000,
        });
        this.dialogRef.close();
      });
    } else {
      body.id = this.drillId;
      this.http
        .put(`/api/DrillRecords/${this.drillId}`, body)
        .subscribe((data: any) => {
          this.isLoadingSave = false;
          this.snackBar.open("保存成功", "确定", {
            verticalPosition: "top",
            duration: 3000,
          });
          this.dialogRef.close();
        });
    }
  }

  //查看图片
  viewImg(url) {
    let dom = document.getElementById(`viewerjs`);
    let pObjs = dom.childNodes;
    let node = document.createElement("img");
    node.style.display = "none";
    node.src = url;
    node.id = "img";
    dom.appendChild(node);
    setTimeout(() => {
      let viewer = new Viewer(document.getElementById(`viewerjs`), {
        hidden: () => {
          dom.removeChild(pObjs[0]);
          viewer.destroy();
        },
      });
      node.click();
    }, 0);
  }
}
