import { Component, OnInit, Inject, ViewChild } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import {
  MatDialog,
  MatDialogRef,
  MAT_DIALOG_DATA,
} from "@angular/material/dialog";
import { MatPaginator } from "@angular/material/paginator";
import { MatSnackBar, MatSnackBarConfig } from "@angular/material/snack-bar";
import Swiper from "swiper";
import { Router, ActivatedRoute } from "@angular/router";

@Component({
  selector: "app-realistic-picture-look",
  templateUrl: "./realistic-picture.component.html",
  styleUrls: ["./realistic-picture.component.scss"],
})
export class RealisticPictureLookComponent implements OnInit {
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private http: HttpClient,
    public dialog: MatDialog,
    public snackBar: MatSnackBar
  ) {}

  ngOnInit(): void {
    let companyId = this.route.snapshot.queryParams.id;
    this.http
      .get("/api/RealityImageGroups", {
        params: {
          companyId: companyId,
        },
      })
      .subscribe((data: any) => {
        this.allRealPicture = data;
        this.allRealPicture.unshift(
          {
            companyId: sessionStorage.getItem("companyId"),
            id: "重点部位",
            name: "重点部位",
            realityImages: null,
          },
          {
            companyId: sessionStorage.getItem("companyId"),
            id: "安全出口",
            name: "安全出口",
            realityImages: null,
          }
        );
        this.selectReal = data[0];
        this.getAllRealPicture();
      });
  }

  companyId: any; //单位编号
  allRealPicture: any = []; //所有实景图文件
  selectReal: any; //选中的实景图文件
  selectRealIndex: number = 0; //选中的实景图文件下标
  allImages: any = []; //实景图文件对应所有的实景图
  isDownload: boolean = false; //是否批量下载
  downloadList: any = []; //选中需要下载的图片

  //分页
  @ViewChild(MatPaginator, { static: true })
  paginator: MatPaginator;
  length: any; //共多少条数据
  pageSize: any = "12"; //每页条数
  pageSizeOptions: number[] = [12]; //设置每页条数
  PageNumber: any; //第几页

  //分页切换
  chagePage(e) {
    this.PageNumber = e.pageIndex + 1;
    this.getAllRealPicture();
  }

  //获取所有实景图分组
  getAllGrouping() {
    let companyId = this.route.snapshot.queryParams.id;
    this.http
      .get("/api/RealityImageGroups", {
        params: {
          companyId: companyId,
        },
      })
      .subscribe((data: any) => {
        this.allRealPicture = data;
        this.allRealPicture.unshift(
          {
            companyId: sessionStorage.getItem("companyId"),
            id: "重点部位",
            name: "重点部位",
            realityImages: null,
          },
          {
            companyId: sessionStorage.getItem("companyId"),
            id: "安全出口",
            name: "安全出口",
            realityImages: null,
          }
        );
        console.log("文件夹列表", this.allRealPicture);
        this.selectReal = data[this.selectRealIndex];
      });
  }

  //获取实景图分组对应实景图
  getAllRealPicture() {
    if (this.selectReal) {
      if (this.selectReal.id == "安全出口") {
        let params = {
          companyId: this.selectReal.companyId,
          PageNumber: this.PageNumber || 1,
          pageSize: this.pageSize,
        };
        this.http
          .get("/api/CompanySafetyExits/GetCompanySafetyExitImages", {
            params: params,
          })
          .subscribe((data: any) => {
            // console.log('当前单位安全出口', data)
            this.allImages = data.items;
            this.length = data.totalCount;
            this.pageSize = this.pageSize;
            this.downloadList = [];
            this.allImages.forEach((element) => {
              //每张图片设置选中状态为false
              element.newImageUrl = `${element.imageUrl}?x-oss-process=image/resize,m_fill,h_170,w_299`; //处理图片URL地址
              element.nameStart = element.name;
              element.checked = false; //图片是否选中布尔值
            });
          });
      } else {
        let data;
        if (this.selectReal.id == "重点部位") {
          data = {
            groupId: this.selectReal.id,
            companyId: this.selectReal.companyId,
            PageNumber: this.PageNumber || 1,
            pageSize: this.pageSize,
          };
        } else {
          data = {
            groupId: this.selectReal.id,
            PageNumber: this.PageNumber || 1,
            pageSize: this.pageSize,
          };
        }
        this.http
          .get("/api/RealityImages", { params: data })
          .subscribe((data: any) => {
            this.allImages = data.items;
            this.length = data.totalCount;
            this.pageSize = data.pageSize;
            this.downloadList = [];
            this.allImages.forEach((element) => {
              //每张图片设置选中状态为false
              if (this.selectReal.id == "重点部位") {
                element.newImageUrl = `${element.imageUrl}?x-oss-process=image/resize,m_fill,h_170,w_299`;
                element.nameStart = element.name;
              } else {
                element.newImageUrl = `/api/Objects/PlanPlatform/${element.imageUrl}?x-oss-process=image/resize,m_fill,h_170,w_299`; //处理图片URL地址
                element.nameStart = element.name.substring(
                  0,
                  element.name.lastIndexOf(".")
                ); //图片名称前缀
              }
              element.checked = false; //图片是否选中布尔值
              element.nameEnd = element.name.substring(
                element.name.lastIndexOf("."),
                element.name.length
              ); //图片名称后缀
            });
          });
      }
    }
  }

  //切换左侧实景图文件
  changeReal(e, index) {
    if (this.selectRealIndex != index) {
      this.selectReal = e;
      this.selectRealIndex = index;
      this.isDownload = false;
      this.downloadList = [];
      this.getAllRealPicture();
    }
  }

  //选择批量下载
  download() {
    this.isDownload = !this.isDownload;
    if (!this.isDownload) {
      //取消批量下载时数组清空
      this.allImages.forEach((element) => {
        element.checked = false;
      });
      this.downloadList = [];
    }
  }

  // 预览图片---批量选择图片
  operation(e, index) {
    if (this.isDownload) {
      //批量选择图片
      e.checked = !e.checked;
      if (e.checked == true) {
        this.downloadList.push(e);
      } else {
        this.downloadList.splice(
          this.downloadList.findIndex((item) => item.id === e.id),
          1
        );
      }
    } else {
      //预览图片
      let data = {
        selectReal: this.selectReal,
        allImages: this.allImages,
        imgIndex: index,
      };
      let dialogRef = this.dialog.open(previewImg3, { data });
      dialogRef.afterClosed().subscribe();
    }
  }

  //新建实景图文件
  addReal() {}

  //编辑实景图文件
  editReal() {}

  //删除实景图文件
  delete() {
    let companyId = this.route.snapshot.queryParams.id;
    if (this.selectReal) {
      let isDelete = confirm("您确定要删除吗");
      if (isDelete) {
        this.http
          .delete(`/api/RealityImageGroups/${this.selectReal.id}`)
          .subscribe((data) => {
            this.http
              .get("/api/RealityImageGroups", {
                params: {
                  companyId: companyId,
                },
              })
              .subscribe((data: any) => {
                this.allRealPicture = data;
                this.selectReal = data[this.selectRealIndex];
                this.allImages = [];
                this.getAllRealPicture();
              });
          });
      }
    }
  }

  //实景图修改
  preservationImg(e) {
    let data = {
      id: e.id,
      name: e.nameStart + e.nameEnd,
      imageUrl: e.imageUrl,
      realityImageGroupId: e.realityImageGroupId,
    };
    this.http.put(`/api/RealityImages/${e.id}`, data).subscribe((data) => {
      this.getAllRealPicture();
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("图片修改成功", "确定", config);
    });
  }

  //实景图删除
  deleteImg(e) {
    let isDelete = confirm("您确定要删除吗");
    if (isDelete) {
      this.http.delete(`/api/RealityImages/${e.id}`).subscribe((data) => {
        this.getAllRealPicture();
        const config = new MatSnackBarConfig();
        config.verticalPosition = "top";
        config.duration = 3000;
        this.snackBar.open("图片删除成功", "确定", config);
      });
      this.http
        .delete(`/api/Objects/PlanPlatform/${e.imageUrl}`)
        .subscribe((data) => {});
    }
  }

  isLoading: boolean = false; //loading加载
  //批量下载↓
  //开始下载
  async batchDownload() {
    if (this.downloadList.length) {
      //开始批量循环下载
      this.isLoading = true;
      for (let i = 0; i < this.downloadList.length; i++) {
        let result = await new Promise((result, reject) => {
          this.http
            .get(`/api/Objects/PlanPlatform/${this.downloadList[i].imageUrl}`, {
              responseType: "blob",
            })
            .subscribe((data) => {
              let url = window.URL.createObjectURL(new Blob([data])); //createObjectURL创建一个下载Blob的url地址
              let link = document.createElement("a");
              link.style.display = "none";
              link.href = url;
              link.setAttribute("download", this.downloadList[i].name);
              document.body.appendChild(link);
              link.click();
              result("success");
            });
        });
        if (i == this.downloadList.length - 1) {
          //判断是否下载完毕
          this.isLoading = false;
          this.isDownload = false;
          this.allImages.forEach((element) => {
            element.checked = false;
          });
          this.downloadList = [];
        }
      } //for循环
    } else {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("请选择图片", "确定", config);
    }
  }
}

//预览图片组件
@Component({
  selector: "app-previewImg",
  templateUrl: "./previewImg.html",
  styleUrls: ["./realistic-picture.component.scss"],
})
export class previewImg3 {
  constructor(
    private http: HttpClient,
    public dialog: MatDialog,
    public dialogRef: MatDialogRef<previewImg3>,
    @Inject(MAT_DIALOG_DATA) public data
  ) {}
  testSwiper: Swiper;

  ngOnInit(): void {
    this.allImages = this.data.allImages;
    this.allImages.forEach((element) => {
      if (
        this.data.selectReal.id == "重点部位" ||
        this.data.selectReal.id == "安全出口"
      ) {
        element.previewImageUrl = `${element.imageUrl}`; //处理图片URL地址
      } else {
        element.previewImageUrl = `/api/Objects/PlanPlatform/${element.imageUrl}?x-oss-process=image/auto-orient,1`; //处理图片URL地址
      }
    });
  }

  ngAfterViewInit() {
    this.testSwiper = new Swiper(".swiper-container", {
      lazy: true,
      initialSlide: this.data.imgIndex,
      direction: "horizontal",
      loop: false,

      // 如果需要前进后退按钮
      navigation: {
        nextEl: ".swiper-button-next",
        prevEl: ".swiper-button-prev",
      },
    });
  }

  allImages: any; //展示所有的图片
  rotationAngle: number = 0; //旋转角度

  //旋转图片
  rotate() {
    this.rotationAngle = this.rotationAngle + 90;
    if (this.rotationAngle === 360) {
      this.rotationAngle = 0;
    }
  }
}
