import { HttpClient } from '@angular/common/http';
import { Component, EventEmitter, NgZone, OnInit, Output } from '@angular/core';
import { MatSnackBar, MatSnackBarConfig } from '@angular/material/snack-bar';
import { ActivatedRoute } from '@angular/router';
import { MapFactory } from '@src/modules/map/declare/factory';
import { IDriving, IMap } from '@src/modules/map/declare/map';
declare var AMap: any;

@Component({
  selector: 'app-router-gis',
  templateUrl: './router-gis.component.html',
  styleUrls: ['./router-gis.component.scss']
})
export class RouterGISComponent implements OnInit {
  @Output() onCustomEvent: EventEmitter<any> = new　EventEmitter();//创建实力
  constructor(public _ngZone:NgZone,public snackBar: MatSnackBar,private http:HttpClient,private route:ActivatedRoute,) { }

  ngOnInit(): void {
  }

  ngAfterViewInit(): void {
    let async = this.getCompanyData()
    window.setTimeout(()=>{
      async.then(res=>{
        this.mapInit()
      })
    },0)
  }

  //获取单位 信息
  getCompanyData () {
    return new Promise((resolve,reject)=>{
      this.http.get(`/api/Companies/${this.route.snapshot.queryParams.id}`).subscribe((data:any)=>{
        if (data.driveRouteStartPoint && data.driveRouteStartPoint.x && data.driveRouteStartPoint.y && data.driveRouteStartName) { //开始坐标 名称
          this.routeStart = data.driveRouteStartName
          this.startCoordinate = MapFactory.LngLatInstance(data.driveRouteStartPoint.x, data.driveRouteStartPoint.y)
        }
        if (data.driveRouteEndPoint && data.driveRouteEndPoint.x && data.driveRouteEndPoint.y && data.driveRouteEndName) { //结束坐标 名称
          this.routeEnd = data.driveRouteEndName
          this.endCoordinate = MapFactory.LngLatInstance(data.driveRouteEndPoint.x, data.driveRouteEndPoint.y)
        } else if (data.location && data.location.x && data.location.y) {
          this.routeEnd = data.name
          this.endCoordinate = MapFactory.LngLatInstance(data.location.x, data.location.y)
        }
        resolve('success')
      }) //http
    })
  }

  //保存 路线
  submitRouter () {
    if (this.startCoordinate && this.endCoordinate && this.routeStart && this.routeEnd) {
      let start = {x:null, y:null}
      let end = {x:null, y:null}
      if (this.startCoordinate instanceof Array) {
        start.x = this.startCoordinate[0]
        start.y = this.startCoordinate[1]
      } else {
        start.x = this.startCoordinate.lng
        start.y = this.startCoordinate.lat
      }
      if (this.endCoordinate instanceof Array) {
        end.x = this.endCoordinate[0]
        end.y = this.endCoordinate[1]
      } else {
        end.x = this.endCoordinate.lng
        end.y = this.endCoordinate.lat
      }
      let params = {
        id: this.route.snapshot.queryParams.id,
        driveRouteStartPoint: start,
        driveRouteStartName: this.routeStart,
        driveRouteEndPoint: end,
        driveRouteEndName: this.routeEnd,
      }
      this.http.put(`/api/Companies/${this.route.snapshot.queryParams.id}`,params).subscribe(data=>{
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('保存成功','确定',config);
      })
    } else {
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('请先规划路线','确定',config);
    }
  }

  //地图初始化
  mapInit () {
    let that = this
    var layer = MapFactory.LayerInstance({
      zooms:[3,20],    //可见级别
      visible:true,    //是否可见
      opacity:1,       //透明度
      zIndex:0,         //叠加层级
      resizeEnable: true //是否监控地图容器尺寸变化,
    })
    that.map = MapFactory.MapInstance('map',{
      layers:[layer], //当只想显示标准图层时layers属性可缺省,
    });
    that.map.setCity('上海市');
    that.startCoordinate && that.endCoordinate? that.queryGISRoute() : null
  }

  map:IMap; //地图实例
  routeStart:any; //起点
  routeStartList:any = [] //起点 搜索结果
  routeEnd:any; //终点
  routeEndList:any = [] //终点 搜索结果
  timeout:any; //延时器
  routeGIS:IDriving; //查询结果  规划路线
  selectType:boolean = true; //路线选择 推荐方案/躲避用拥堵
  routes:any = { distance: 0, time: 0, steps: [], };//导航查询结果 路线规划
  startCoordinate:any//开始坐标 
  endCoordinate:any//结束坐标

  //输入框 输入事件
  routeChange (e) {
    let that = this
    window.clearTimeout(this.timeout)
    this.timeout = window.setTimeout(()=>{

      that.map.getCity( function(info){ //获取当前 city
        var autoOptions = {city: info.city} 
        let keywords
        e == 0 ? keywords = that.routeStart : keywords = that.routeEnd
        var autoComplete = MapFactory.AutocompleteInstance(autoOptions,that);
        autoComplete.search(keywords, function(status, result) {
          if (result && result.tips && result.tips.length) { //搜索到数据时
            that._ngZone.run(()=>{ 
              if(e == 0){
                that.routeStartList = result.tips
                if(result.tips.length != 0){
                  for (let index = 0; index < result.tips.length; index++) {
                    const element = result.tips[index];
                    if(element.location){
                      that.startCoordinate = MapFactory.LngLatInstance(element.location.lng, element.location.lat)
                      return
                    }
                  }
                }
              }else{
                that.routeEndList = result.tips 
                that.endCoordinate = MapFactory.LngLatInstance(result.tips[0].location.lng, result.tips[0].location.lat)
              }
            });
          } //if
        })

      }); //获取当前 city 

    },500)
  }

  //清空导航输入框
  deleteSearchGIS (e) {
    if (e==0) {
      this.routeStart = null
      this.routeStartList = []
      this.startCoordinate = null
    } else {
      this.routeEnd = null
      this.routeEndList = []
      this.endCoordinate = null
    }
  }

  //清除路线
  clearGISRoute () {
    this.routeStart = null
    this.routeStartList = []
    this.routeEnd = null
    this.routeEndList = []
    this.startCoordinate = null
    this.endCoordinate = null
    this.routeGIS? this.routeGIS.clear() : null
    this.routes = { distance: 0, time: 0, steps: [], }
    this.map.setCity('上海市'); //设置地图中心点
  }

  //交换起始点
  exchangeGISRoute () {
    let data = this.routeStart
    this.routeStart = this.routeEnd
    this.routeEnd = data
    this.routeStartList = []
    this.routeEndList = []
    let data2 = this.startCoordinate
    this.startCoordinate = this.endCoordinate
    this.endCoordinate = data2
  }

  //导航查询
  queryGISRoute () {
    if (this.startCoordinate && this.endCoordinate) {
      //构造路线导航类
      let that = this
      this.selectType = true
      this.routeGIS? this.routeGIS.clear() : null
      this.routeGIS = MapFactory.DrivingInstance({
        map: this.map,
      });
      if(!this.startCoordinate){
        let config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('未查询到起点坐标信息，请输入有效地址','确定',config);
        return
      }else if(!this.startCoordinate){
        let config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('未查询到终点坐标信息，请输入有效地址','确定',config);
        return
      }else{
        this.routeGIS.search(this,this.startCoordinate,this.endCoordinate,
          function(status, result) {
            if (status === 'complete') {
              that.routes = result.routes[0]
            } else { alert('获取驾驶规划路线失败') }
          }
        );
      }
    } else {
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('请选择起始点','确定',config);
    }
  }

  //切换 导航模式 推荐方案/躲避拥堵
  toggleRoute (e) {
    let that = this
    if (this.selectType != e) {
      this.selectType = e
      this.routeGIS? this.routeGIS.clear() : null
      this.routeGIS = MapFactory.DrivingInstance({
        map: this.map,
        policy: e==true? AMap.DrivingPolicy.LEAST_TIME : AMap.DrivingPolicy.REAL_TRAFFIC
      }); 
      // 根据起终点名称规划驾车导航路线
      this.routeGIS.search(this,this.startCoordinate,this.endCoordinate,
        function(status, result) {
          if (status === 'complete') {
            that.routes = result.routes[0]
          } else { alert('获取驾驶规划路线失败') }
        }
      );
    }
  }

}
