import { Component, OnInit, ViewChild, Inject } from '@angular/core';
import { HttpClient } from '@angular/common/http'
import { MatTreeFlatDataSource, MatTreeFlattener } from '@angular/material/tree';
import { MatPaginator } from '@angular/material/paginator';
import { FlatTreeControl } from '@angular/cdk/tree';
import { FormControl } from '@angular/forms';
import { Router,ActivatedRoute } from '@angular/router'
import { PageEvent } from '@angular/material/paginator';
import { MatDialogRef, MatDialog, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { MatSnackBar, MatSnackBarConfig } from '@angular/material/snack-bar';
import { TreeService } from '../../http-interceptors/tree.service'



export interface Food {
  name:string;
  value: string;
}
@Component({
  selector: 'app-plan-audit',
  templateUrl: './plan-audit.component.html',
  styleUrls: ['./plan-audit.component.scss']
})
export class PlanAuditComponent implements OnInit {

  constructor(private http:HttpClient,private router:Router,private route:ActivatedRoute,private tree: TreeService,public dialog: MatDialog,public snackBar: MatSnackBar) { }

  private _transformer = (node, level: number) => { //初始化tree
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      id: node.id,
      parentId: node.parentId,
      children: node.children
    };
  }
  treeControl = new FlatTreeControl<any>(node => node.level, node => node.expandable);
  treeFlattener = new MatTreeFlattener(this._transformer, node => node.level, node => node.expandable, node => node.children);
  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  myControl = new FormControl();
  hasChild = (_: number, node: any) => node.expandable;

  organizationName:any //当前单位所属辖区中队名称
  allunittype:any //所有单位类型
  preparelevels:Food[] //编制级别
  planpatterns:Food[] = [  //预案类型
    {name:"卡片预案",value:"8"},
    {name:"二维预案",value:"1" },
    {name:"三维预案",value:"2" },
    {name:"其他预案",value:"4" },]
  passstates:Food[]=[ //审核状态
    {name:"审核中",value:"1"},
    {name:"审核通过",value:"2"},
    {name:"审核退回",value:"4"},]

  displayedColumns: string[] = ['unitname','planname', 'addpeople', 'addtime','plantype','auditstate', 'isopen','preparethelevel','operation'];
  tabledataSource:any; //所有预案审核信息

  //分页
  @ViewChild(MatPaginator, {static: true})
  pageEvent: PageEvent;
  paginator: MatPaginator;
  length:any; //共多少条数据
  pageSize:any; //每页条数
  pageSizeOptions: number[] = [10] //设置每页条数
  PageNumber:number = 1; //第几页

  //查询筛选条件
  CompanyName:any; //单位名称
  selectOrganizationID:any //选中的辖区中队id
  selectOrganizationIName:any //选中的辖区中队name
  HasChildrenOrganization:boolean = false; //辖区中队是否包含下级
  BuildingTypeId:any; //选中的单位类型
  PlanType:any; //选中的预案类型
  AuditStatus:any = '1'; //选中的审核状态
  PlanLevel:any; //选中的编制级别
  HasChildrenPlanLevel:boolean = false; //编制级别是否包含下级

  //分页事件
  chagePage(e){
    this.PageNumber = e.pageIndex + 1
    let header = {
      CompanyName: this.CompanyName || '',
      OrganizationId: this.selectOrganizationID || '',
      HasChildrenOrganization: String(this.HasChildrenOrganization) || 'false',
      BuildingTypeId: this.BuildingTypeId || '',
      PlanType: this.PlanType || '',
      AuditStatus: this.AuditStatus || '1',
      PlanLevel: this.PlanLevel || '',
      HasChildrenPlanLevel: String(this.HasChildrenPlanLevel) || 'false',
      PageNumber: String(this.PageNumber),
    }
    this.http.get('/api/PlanAudits',{params:header}).subscribe((data:any)=>{
      this.length = data.totalCount
      this.pageSize = data.pageSize
      this.tabledataSource = data.items
    })
  }

  //查询
  onSubmit(e){
    this.getAllPlanAudits()
  }

  //重置
  reset () {
    this.CompanyName = ''
    this.selectOrganizationID = ''
    this.selectOrganizationIName = ''
    this.HasChildrenOrganization = false
    this.BuildingTypeId = ''
    this.PlanType = '',
    this.AuditStatus = '1',
    this.PlanLevel = '',
    this.HasChildrenPlanLevel = false
    this.getAllPlanAudits()
  }

  //获取当前所有预案审核
  getAllPlanAudits () {
    let header = {
      CompanyName: this.CompanyName || '',
      OrganizationId: this.selectOrganizationID || '',
      HasChildrenOrganization: String(this.HasChildrenOrganization) || 'false',
      BuildingTypeId: this.BuildingTypeId || '',
      PlanType: this.PlanType || '',
      AuditStatus: this.AuditStatus || '1',
      PlanLevel: this.PlanLevel || '',
      HasChildrenPlanLevel: String(this.HasChildrenPlanLevel) || 'false',
      PageNumber: '1',
    }
    this.http.get('/api/PlanAudits',{params:header}).subscribe((data:any)=>{
      this.length = data.totalCount
      this.pageSize = data.pageSize
      this.pageEvent.pageIndex = 0
      this.tabledataSource = data.items
    })
  }

  ngOnInit(): void {
    this.getunitdata();
    this.getOrganizations();
    this.getUnittype();
    let level = sessionStorage.getItem("level");
    if(level == "0"){ //如果是总队
      this.preparelevels = [
        {name:"总队",value:"1"},
        {name:"支队",value:"2"},
        {name:"大队",value:"4"},
        {name:"中队",value:"8"},] }
    if(level == "1"){ //如果是支队
      this.preparelevels = [
        {name:"支队",value:"2"},
        {name:"大队",value:"4"},
        {name:"中队",value:"8"},] }
    if(level == "2"){ //如果是大队
      this.preparelevels = [
        {name:"大队",value:"4"},
        {name:"中队",value:"8"},] }
    if(level == "3"){ //如果是中队
      this.preparelevels = [{name:"中队",value:"8"}] }

    this.getAllPlanAudits()
  }

  //得到当前账号所属辖区中队名称
  getunitdata(){
    this.http.get("/api/Account/Profiles").subscribe((data:any)=>{
      this.organizationName = data.organizationName
    })
  }

  //获得所有辖区中队(处理数据结构)
  getOrganizations(){
    let treeData = []
    this.http.get('/api/Organizations').subscribe((data:any)=>{
      data.forEach(element => {
        element.children = []
        data.forEach(item => { if (item.parentId == element.id) {element.children.push(item)} });
      });
      if (this.organizationName) { //当前账号有辖区中队时
        data.forEach(element => {
          if (element.name===this.organizationName) {
            treeData.push(element)
            this.dataSource.data = treeData }
        });
      } else { //当前账号没有有辖区中队时
        data.forEach(element => {
          if (!element.parentId) {
            treeData.push(element)
            this.dataSource.data = treeData }
        });
      }

    })
  }

  //获得所有单位类型
  getUnittype(){
    this.http.get('/api/BuildingTypes/Simple').subscribe(data=>{ 
      this.allunittype = data
    })
  }

  isorganizationbox:boolean = false //辖区中队DIV显隐

  //打开辖区中队
  openorganizationbox() {
    this.isorganizationbox = true
  }

  //关闭辖区中队
  closediv(){
    this.isorganizationbox = false
  }

  //选择辖区中队时
  add(node) {
    this.selectOrganizationID = node.id
    this.selectOrganizationIName = node.name
    this.isorganizationbox = false
  }

  //预案审核
  toExamine (e) {
    window.open(`/planAudit/planpass?id=${e.companyId}&companyName=${e.companyName}&auditPlanId=${e.id}&auditStatus=${e.auditStatus}&planType=1`);
  }

  //预案公开
  openReserve (e) {
    this.http.put(`/api/PlanAudits/${e.id}/Public`,[]).subscribe(data=>{
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('预案已公开','确定',config);
      this.renew()
    })
  }

  //预案取消公开
  closeReserve (e) {
    this.http.put(`/api/PlanAudits/${e.id}/Unpublic`,[]).subscribe(data=>{
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('预案已取消公开','确定',config);
      this.renew()
    })
  }

  //按查询条件及分页 更新当前页面
  renew () {
    let header = {
      CompanyName: this.CompanyName || '',
      OrganizationId: this.selectOrganizationID || '',
      HasChildrenOrganization: String(this.HasChildrenOrganization) || 'false',
      BuildingTypeId: this.BuildingTypeId || '',
      PlanType: this.PlanType || '',
      AuditStatus: this.AuditStatus || '1',
      PlanLevel: this.PlanLevel || '',
      HasChildrenPlanLevel: String(this.HasChildrenPlanLevel) || 'false',
      PageNumber: String(this.PageNumber),
    }
    this.http.get('/api/PlanAudits',{params:header}).subscribe((data:any)=>{
      this.length = data.totalCount
      this.pageSize = data.pageSize
      this.tabledataSource = data.items
    })
  }



}



@Component({
  selector: 'planopen',
  templateUrl: './plan-opendialog.html',
  styleUrls: ['./plan-audit.component.scss']
})
export class PlanOpen {
  
  constructor(private http: HttpClient,public dialogRef: MatDialogRef<PlanOpen>,@Inject(MAT_DIALOG_DATA) public data,public snackBar: MatSnackBar,private tree: TreeService,
  private route:ActivatedRoute,) {}

  ngOnInit(): void {

  }

  //审核预案
  onSubmit (e) {
    e.auditStatus = Number(e.auditStatus)
    this.http.put(`/api/PlanAudits/${this.data}`,e).subscribe(data=>{
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('审批结果提交成功','确定',config);
      this.dialogRef.close();
    })
  }

  //提醒弹窗
  remind () {
    let type = this.route.snapshot.queryParams.auditStatus //当前预案审核状态 =2为审核通过时
    if (type==2) {
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('预案驳回,公开预案会设置为未公开状态','确定',config);
    }
  }



}