import { Component, OnInit, ViewChild } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { MatPaginator } from "@angular/material/paginator";
import { PageEvent } from "@angular/material/paginator";
import { MatDialog } from "@angular/material/dialog";
import { MatSnackBar } from "@angular/material/snack-bar";
import { DatePipe } from "@angular/common";
import { AddPlanComponent } from "./add-plan/add-plan.component";

@Component({
  selector: "app-plan-deduction",
  templateUrl: "./plan-deduction.component.html",
  styleUrls: ["./plan-deduction.component.scss"],
  providers: [DatePipe],
})
export class PlanDeductionComponent implements OnInit {
  constructor(
    private http: HttpClient,
    public dialog: MatDialog,
    public snackBar: MatSnackBar,
    private datePipe: DatePipe
  ) {}

  //分页
  @ViewChild(MatPaginator, { static: true })
  pageEvent: PageEvent;
  paginator: MatPaginator;
  displayedColumns: string[] = [
    "planname",
    "name",
    "people",
    "time",
    "operate",
  ];
  tabledataSource: any; //表格数据

  ngOnInit(): void {
    this.getOrganizations();
  }

  //获得所有组织机构
  allorganizations: any; //所有组织机构
  getOrganizations() {
    this.http.get("/api/Organizations").subscribe((data: any) => {
      this.allorganizations = data;
      this.getpresentOrganization();
    });
  }

  //得到当前单位所在组织机构的tree型数据
  expandKeys = [];
  nodes = [];
  organizationId;
  getpresentOrganization() {
    this.allorganizations.forEach((item) => {
      item.key = item.id;
      item.title = item.name;
      item.children = [];
      this.allorganizations.forEach((element) => {
        if (element.parentId == item.id) {
          item.children.push(element);
        }
        if (item.children.length === 0) {
          item.isLeaf = true;
        } else {
          item.isLeaf = false;
        }
      });
    });
    this.http.get("/api/Account/Profiles").subscribe((data: any) => {
      this.allorganizations.forEach((item) => {
        if (item.name == data.organizationName) {
          this.expandKeys = [item.id];
          this.organizationId = item.id;
          // this.orId = item.id;
          this.nodes = [...[item]];
          this.getTabledataSource();
        }
      });
    });
  }

  length: any; //共多少条数据
  pageSize: any = 10; //每页条数
  PageNumber: any = 1; //第几页
  getTabledataSource() {
    let params: any = {
      Name: this.name || "",
      CompanyName: this.companyname || "",
      CommitterOrganizationId: this.orId || "",
      PlanCategory: 9,
      PageNumber: this.PageNumber,
      PageSize: this.pageSize,
    };
    this.http
      .get("/api/PlanComponents/GetAll", { params: params })
      .subscribe((data: any) => {
        console.log("预案数据", data);
        this.tabledataSource = data.items;
        this.length = data.totalCount;
      });
  }

  //分页事件
  chagePage(e) {
    this.PageNumber = e.pageIndex + 1;
    this.getTabledataSource();
  }

  name;
  companyname;
  orId;
  time;
  //查询
  onSubmit() {
    this.PageNumber = 1;
    this.pageEvent.pageIndex = 0;
    this.getTabledataSource();
  }

  //重置
  reset() {
    this.name = "";
    this.companyname = "";
    this.orId = "";
    this.onSubmit();
  }

  add() {
    const dialogRef = this.dialog.open(AddPlanComponent, {
      width: "400px",
      data: this.nodes,
    });
    dialogRef.afterClosed().subscribe((data) => {
      if (data) {
        console.log("创建成功", data);
        this.onSubmit();
        sessionStorage.setItem("planId", data.id);
        sessionStorage.setItem("companyId", data.plan.companyId);
        sessionStorage.setItem(
          "buildingTypeId",
          data.plan.company.buildingTypes[0].id
        );
        sessionStorage.setItem("editable", "1");
        let orId = data.plan.company.organizationId;
        let orName = data.plan.company.organizationName;
        window.open(
          `/keyUnit/editunitinfo?id=${data.plan.company.id}&orName=${orName}&orId=${orId}`
        );
      }
    });
  }

  look(element) {
    console.log(element);
    sessionStorage.setItem("planId", element.id);
    sessionStorage.setItem("companyId", element.companyId);
    sessionStorage.setItem(
      "buildingTypeId",
      element.company.buildingTypes[0].id
    );
    sessionStorage.setItem("editable", "1");
    let orId = element.company.organizationId;
    let orName = element.company.organizationName;
    window.open(
      `/keyUnit/editunitinfo?id=${element.companyId}&orName=${orName}&orId=${orId}`
    );
  }

  delete(element) {
    if (confirm("确认删除此预案吗")) {
      this.http
        .delete(`/api/PlanComponents2D/${element.id}`)
        .subscribe((data: any) => {
          alert("删除成功");
          this.getTabledataSource();
        });
    }
  }
}
