import { Component, OnInit, ViewChild, Inject } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { MatPaginator } from "@angular/material/paginator";
import { Router, ActivatedRoute } from "@angular/router";
import { PageEvent } from "@angular/material/paginator";
import { MatDialog } from "@angular/material/dialog";
import { MatSnackBar } from "@angular/material/snack-bar";
import { TreeService } from "../../http-interceptors/tree.service";
import { DatePipe } from "@angular/common";
import { SixFamiliarizeComponent } from "../entry-plan-look/six-familiarize/six-familiarize.component";

@Component({
  selector: "app-six-familiarity",
  templateUrl: "./six-familiarity.component.html",
  styleUrls: ["./six-familiarity.component.scss"],
  providers: [DatePipe],
})
export class SixFamiliarityComponent implements OnInit {
  constructor(
    private http: HttpClient,
    public dialog: MatDialog,
    public snackBar: MatSnackBar,
    private datePipe: DatePipe
  ) {}

  //分页
  @ViewChild(MatPaginator, { static: true })
  pageEvent: PageEvent;
  paginator: MatPaginator;
  displayedColumns: string[] = [
    "planname",
    "name",
    "or",
    "people",
    "time",
    "operate",
  ];
  tabledataSource: any; //表格数据

  ngOnInit(): void {
    this.getOrganizations();
  }

  //获得所有组织机构
  allorganizations: any; //所有组织机构
  getOrganizations() {
    this.http.get("/api/Organizations").subscribe((data: any) => {
      this.allorganizations = data;
      this.getpresentOrganization();
    });
  }

  //得到当前单位所在组织机构的tree型数据
  expandKeys = [];
  nodes = [];
  organizationId;
  getpresentOrganization() {
    this.allorganizations.forEach((item) => {
      item.key = item.id;
      item.title = item.name;
      item.children = [];
      this.allorganizations.forEach((element) => {
        if (element.parentId == item.id) {
          item.children.push(element);
        }
        if (item.children.length === 0) {
          item.isLeaf = true;
        } else {
          item.isLeaf = false;
        }
      });
    });
    this.http.get("/api/Account/Profiles").subscribe((data: any) => {
      this.allorganizations.forEach((item) => {
        if (item.name == data.organizationName) {
          this.expandKeys = [item.id];
          this.organizationId = item.id;
          this.orId = item.id;
          this.nodes = [...[item]];
          this.getTabledataSource();
        }
      });
    });
  }

  length: any; //共多少条数据
  pageSize: any = 10; //每页条数
  PageNumber: any = 1; //第几页
  getTabledataSource() {
    let params = {
      OrganizationId: this.orId,
      PlanComponentName: this.name || "",
      StartTime: this.time
        ? this.datePipe.transform(this.time[0], "yyyy-MM-ddT00:00:00")
        : "2020-01-01T00:00:00",
      EndTime: this.time
        ? this.datePipe.transform(this.time[1], "yyyy-MM-ddT23:59:59")
        : "2030-01-01T00:00:00",
      PageNumber: this.PageNumber,
      PageSize: this.pageSize,
    };
    this.http
      .get("/api/SixFamiliars", { params: params })
      .subscribe((data: any) => {
        this.tabledataSource = data.items;
        this.length = data.totalCount;
      });
  }

  //分页事件
  chagePage(e) {
    this.PageNumber = e.pageIndex + 1;
    this.getTabledataSource();
  }

  name;
  companyname;
  orId;
  time;
  //查询
  onSubmit() {
    this.PageNumber = 1;
    this.pageEvent.pageIndex = 0;
    this.getTabledataSource();
  }

  //重置
  reset() {
    this.name = "";
    this.orId = this.organizationId;
    this.time = null;
    this.onSubmit();
  }

  look(element) {
    console.log(element);
    let unitdata = {
      unitname: element.planComponent.name,
    };
    this.dialog.open(SixFamiliarizeComponent, {
      width: "700px",
      data: {
        planData: element,
        unitdata: unitdata,
        isEditMode: false,
        id: element.planComponentId,
      },
    });
  }

  delete(element) {
    if (confirm("确认删除此记录吗")) {
      this.http
        .delete(`/api/SixFamiliars/${element.id}`)
        .subscribe((data: any) => {
          alert("删除成功");
          this.getTabledataSource();
        });
    }
  }
}
