import { Component, OnInit, ViewChild, Inject } from '@angular/core';
import {HttpClient} from '@angular/common/http'
import { MatDialogRef, MatDialog, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { PageEvent } from '@angular/material/paginator';
import { AddEnterpriserUser } from './addenterpriseuser.component'
import { MatSnackBar, MatSnackBarConfig } from '@angular/material/snack-bar';



@Component({
  selector: 'app-enterpriseuser',
  templateUrl: './enterpriseuser.component.html',
  styleUrls: ['./enterpriseuser.component.scss']
})
export class EnterpriseuserComponent implements OnInit {

  constructor(public http: HttpClient,public dialog: MatDialog,public snackBar: MatSnackBar) { }
  
  ngOnInit() {
    this.initData()
  }

  displayedColumns: string[] = ['loginName', 'unitName', 'creditcode','time', 'operation'];
  dataSource:any;   //所有企业用户

  userLogin:string; //搜索账号
  userName:string; //搜索名称
  creditcode:string//统一社会信用代码
  //分页
  @ViewChild(MatPaginator, {static: true}) 
  pageEvent: PageEvent;
  paginator: MatPaginator;
  length:any; //共多少条数据
  pageSize:any; //每页条数
  pageSizeOptions: number[] = [10] //设置每页条数
  pageNumber:number = 1; //第几页

  //分页切换
  chagePage (e) {
    this.pageNumber = e.pageIndex+1
    let data= {
      Name: this.userLogin || '',
      CompanyName: this.userName || '',
      USCI: this.creditcode || '',
      PageNumber: String(this.pageNumber),
    }
    this.http.get('/api/CompanyUsers',{params:data}).subscribe((data:any)=>{
      this.length = data.totalCount
      this.pageSize = data.pageSize
      this.dataSource = new MatTableDataSource<any>(data.items)
    })
  }

  //页面初始化 + 查询 + 重置
  initData () {
    let data= {
      Name: this.userLogin || '',
      CompanyName: this.userName || '',
      USCI: this.creditcode || '',
      PageNumber: '1',
    }
    this.http.get('/api/CompanyUsers',{params:data}).subscribe((data:any)=>{
      this.length = data.totalCount
      this.pageSize = data.pageSize
      this.pageEvent.pageIndex = 0
      this.dataSource = new MatTableDataSource<any>(data.items)
    })
  }

  //更新当前页数据
  getAllCompanyUsers () {
    let data= {
      Name: this.userLogin || '',
      CompanyName: this.userName || '',
      USCI: this.creditcode || '',
      PageNumber: String(this.pageNumber),
    }
    this.http.get('/api/CompanyUsers',{params:data}).subscribe((data:any)=>{
      this.length = data.totalCount
      this.pageSize = data.pageSize
      this.dataSource = new MatTableDataSource<any>(data.items)
    })
  }

  //清空搜索
  empty () {
    this.userLogin = ''
    this.userName = ''
    this.creditcode = ''
    this.initData()
  }
  
  //创建企业用户
  open(){
    let dialogRef = this.dialog.open(AddEnterpriserUser);
    dialogRef.afterClosed().subscribe(data=>{
      if (data) {this.getAllCompanyUsers()}
    });
  }

  //编辑企业用户
  edit (e) {
    let dialogRef = this.dialog.open(editenterpriseuser,{data:e});
    dialogRef.afterClosed().subscribe(data=>{
      if (data) {this.getAllCompanyUsers()}
    });
  }

  //重置密码
  reset (e) {
    this.http.put(`/api/CompanyUsers/${e.name}/ResetPassword`,{}).subscribe(data=>{
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('重置密码成功!','确定',config);
    })
  }

  //查看企业信息
  see (e) {
    this.http.get(`/api/CompanyUsers/${e.name}`).subscribe(
      data=> {
        let dialogRef = this.dialog.open(seeenterpriseuser, {data});
        dialogRef.afterClosed().subscribe(); 
      }
    )
  }

  //启用
  enabled (e) {
    this.http.put(`/api/CompanyUsers/${e.name}`,{
      name:e.name,
      phone:e.phone,
      enabled:true,
      creationTime:e.creationTime,
      usci:e.usci,
      companyId:e.companyId,
      companyName:e.companyName
    }).subscribe(data=>{
      this.getAllCompanyUsers()
    })
  }

  //禁用
  noEnabled (e) {
    this.http.put(`/api/CompanyUsers/${e.name}`,{
      name:e.name,
      phone:e.phone,
      enabled:false,
      creationTime:e.creationTime,
      usci:e.usci,
      companyId:e.companyId,
      companyName:e.companyName
    }).subscribe(data=>{
      this.getAllCompanyUsers()
    })
  }

  //注销用户
  logoff (e) {
    let isTrue = confirm('注销是不可逆操作,您确定要注销吗')
    if (isTrue) {
      this.http.delete(`/api/CompanyUsers/${e.name}/Cancel`).subscribe(data=>{
        this.getAllCompanyUsers() })
    }
  }

  //删除
  delete (e) {
    let isTrue = confirm('您确定要删除吗')
    if (isTrue) {
      this.http.delete(`/api/CompanyUsers/${e.name}`).subscribe(data=>{
        this.getAllCompanyUsers()
      })
    }
  }



}



//编辑企业用户
@Component({
  selector: 'app-editenterpriseuser',
  templateUrl: './editenterpriseuser.html',
  styleUrls: ['./enterpriseuser.component.scss']
})
export class editenterpriseuser {

  constructor(public http: HttpClient,public dialog: MatDialog,
  @Inject(MAT_DIALOG_DATA) public data,public dialogRef: MatDialogRef<any>,) { }
  
  ngOnInit() {
    this.companyName = this.data.companyName
    this.companyPhone = this.data.phone
    this.companyUsci = this.data.usci
  }

  companyName:any; //企业单位名称
  companyPhone:any; //企业电话
  companyUsci:any; //企业统一社会信用代码

  //提交表单
  onSubmit (e) {
    this.http.put(`/api/CompanyUsers/${this.data.name}`,{
      name:this.data.name,
      phone:e.tel,
      enabled:this.data.enabled,
      creationTime:this.data.creationTime,
      usci:e.creditcode,
      companyId:this.data.companyId,
      companyName:e.unitname
    }).subscribe(data=>{
      this.dialogRef.close('success')
    })
  }



}



//查看企业用户
@Component({
  selector: 'app-seeenterpriseuser',
  templateUrl: './seeenterpriseuser.html',
  styleUrls: ['./enterpriseuser.component.scss']
})
export class seeenterpriseuser {

  constructor(public http: HttpClient,public dialog: MatDialog,
  @Inject(MAT_DIALOG_DATA) public data) { }
  
  ngOnInit() {}

}