import { Component, OnInit, Inject } from "@angular/core";
import {
  MatTreeFlatDataSource,
  MatTreeFlattener,
} from "@angular/material/tree";
import { FlatTreeControl } from "@angular/cdk/tree";
import {
  MatDialog,
  MatDialogRef,
  MAT_DIALOG_DATA,
} from "@angular/material/dialog";
import { FormControl } from "@angular/forms";
import { HttpClient } from "@angular/common/http";
import { TreeService } from "../../http-interceptors/tree.service";
import { MatSnackBar, MatSnackBarConfig } from "@angular/material/snack-bar";

@Component({
  selector: "app-navmenus",
  templateUrl: "./navmenus.component.html",
  styleUrls: ["./navmenus.component.scss"],
})
export class NavmenusComponent implements OnInit {
  newdata = [];
  private _transformer = (node, level: number) => {
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      id: node.id,
      parentId: node.parentId,
      children: node.children,
      url: node.url,
      icon: node.icon,
    };
  };
  treeControl = new FlatTreeControl<any>(
    (node) => node.level,
    (node) => node.expandable
  );
  treeFlattener = new MatTreeFlattener(
    this._transformer,
    (node) => node.level,
    (node) => node.expandable,
    (node) => node.children
  );
  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  constructor(
    private http: HttpClient,
    public dialog: MatDialog,
    private tree: TreeService,
    public snackBar: MatSnackBar
  ) {}
  getlist = (): void => {
    this.http.get("/api/NavMenus").subscribe((data: any[]) => {
      this.newdata = this.tree.toTree(data);
      const nodes = this.treeControl.dataNodes;
      const expandNodes = [];
      nodes.forEach((item) => {
        if (item.expandable && this.treeControl.isExpanded(item)) {
          expandNodes.push(item.id);
        }
      });
      this.dataSource.data = this.newdata;
      let newNodes = this.treeControl.dataNodes;
      newNodes = newNodes.filter((n) => {
        return expandNodes.indexOf(n.id) >= 0;
      });
      newNodes.forEach((item) => {
        this.treeControl.expand(item);
      });
    });
  };
  treedata: any;
  //初始化页面时渲染出tree
  ngOnInit() {
    this.http.get("/api/NavMenus").subscribe((data: any[]) => {
      // console.log(111,this.data)
      // this.treedata = this.tree.toTree(data)
      let _data = this.tree.toTree(data);
      this.dataSource.data = _data;
      this.treedata = _data;
    });
  }
  hasChild = (_: number, node: any) => node.expandable;

  //弹出创建窗口按钮
  createauthority(value) {
    const dialogRef = this.dialog.open(CreateMenus, {
      //调用open方法打开对话框并且携带参数过去
      width: "260px",
      data: {
        id: value.id,
        children: value.children,
        icon: value.icon,
        url: value.url,
      },
    });
    dialogRef.afterClosed().subscribe((data) => {
      this.newdata = [];
      this.getlist();
    });
  }
  //删除按钮
  deleted(authority) {
    var isdeleted = confirm("确定要删除此用户吗？");
    if (isdeleted) {
      //请求删除接口
      this.newdata = [];
      this.http.delete(`/api/NavMenus/${authority.id}`).subscribe((data) => {
        this.getlist();
      });
    }
  }
  //编辑按钮
  edit(value) {
    const dialogRef = this.dialog.open(EditMenus, {
      //调用open方法打开对话框并且携带参数过去
      width: "260px",
      data: {
        id: value.id,
        parentId: value.parentId,
        order: value.order,
        node: value,
      },
    });
    dialogRef.afterClosed().subscribe((data) => {
      this.newdata = [];
      this.getlist();
    });
  }
}

//创建组件
@Component({
  selector: "createmenus",
  templateUrl: "./createmenus.component.html",
  styleUrls: ["./navmenus.component.scss"],
})
export class CreateMenus {
  myControl = new FormControl();
  //注入MatDialogRef，可以用来关闭对话框
  //要访问对话框组件中的数据，必须使用MAT_DIALOG_DATA注入令牌
  constructor(
    private http: HttpClient,
    public dialogRef: MatDialogRef<CreateMenus>,
    public snackBar: MatSnackBar,
    @Inject(MAT_DIALOG_DATA) public data
  ) {}

  onNoClick(): void {
    this.dialogRef.close();
  }
  onSubmit(value) {
    // this.http
    //   .post("/api/NavMenus", {
    //     id: "",
    //     name: "预案熟悉",
    //     icon: "featured_play_list",
    //     // url: value.url,
    //     order: 4,
    //     parentId: null,
    //   })
    //   .subscribe(
    //     (data) => {
    //       this.dialogRef.close();
    //     },
    //     (err) => {
    //       const config = new MatSnackBarConfig();
    //       config.verticalPosition = "top";
    //       config.duration = 3000;
    //       this.snackBar.open("请填写正确格式", "确定", config);
    //     }
    //   );
    // return;
    if (this.data.children) {
      this.http
        .post("/api/NavMenus", {
          id: "",
          name: value.name,
          icon: value.icon,
          url: value.url,
          order: this.data.children[this.data.children.length - 1].order + 1,
          parentId: this.data.id,
        })
        .subscribe(
          (data) => {
            this.dialogRef.close();
          },
          (err) => {
            const config = new MatSnackBarConfig();
            config.verticalPosition = "top";
            config.duration = 3000;
            this.snackBar.open("请填写正确格式", "确定", config);
          }
        );
    } else {
      this.http
        .post("/api/NavMenus", {
          id: "",
          name: value.name,
          icon: value.icon,
          url: value.url,
          order: 0,
          parentId: this.data.id,
        })
        .subscribe(
          (data) => {
            this.dialogRef.close();
          },
          (err) => {
            const config = new MatSnackBarConfig();
            config.verticalPosition = "top";
            config.duration = 3000;
            this.snackBar.open("请填写正确格式", "确定", config);
          }
        );
    }
  }
}

//编辑组件
@Component({
  selector: "editmenus",
  templateUrl: "./editmenus.component.html",
  styleUrls: ["./navmenus.component.scss"],
})
export class EditMenus {
  myControl = new FormControl();
  //注入MatDialogRef，可以用来关闭对话框
  //要访问对话框组件中的数据，必须使用MAT_DIALOG_DATA注入令牌
  constructor(
    private http: HttpClient,
    public dialogRef: MatDialogRef<EditMenus>,
    public snackBar: MatSnackBar,
    @Inject(MAT_DIALOG_DATA) public data
  ) {}
  menuname: any; //菜单名称
  menuiconurl: any; //菜单图标地址
  menuweburl: any; //菜单地址
  ngOnInit(): void {
    this.menuname = this.data.node.name;
    this.menuiconurl = this.data.node.icon;
    this.menuweburl = this.data.node.url;
  }
  onNoClick(): void {
    this.dialogRef.close();
  }
  onSubmit(value) {
    if (value.icon) {
      this.data.icon = value.icon;
    }
    if (value.url) {
      this.data.url = value.url;
    }
    this.http
      .put(` /api/NavMenus/${this.data.id}`, {
        id: this.data.id,
        name: value.name,
        icon: this.data.icon,
        url: this.data.url,
        order: this.data.order,
        parentId: this.data.parentId,
      })
      .subscribe(
        (data) => {
          this.dialogRef.close();
        },
        (err) => {
          const config = new MatSnackBarConfig();
          config.verticalPosition = "top";
          config.duration = 3000;
          this.snackBar.open("请填写正确格式", "确定", config);
        }
      );
  }
}
