import { Component, OnInit, ViewChild, Inject } from '@angular/core';
import {HttpClient} from '@angular/common/http'
import { MatDialogRef, MatDialog, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { FlatTreeControl } from '@angular/cdk/tree';
import { MatTreeFlattener, MatTreeFlatDataSource } from '@angular/material/tree';
import { PageEvent } from '@angular/material/paginator';
import { MatSnackBar, MatSnackBarConfig } from '@angular/material/snack-bar';
import { TreeService } from '../../http-interceptors/tree.service'
import { FormControl } from '@angular/forms';



@Component({
  selector: 'app-usermanagement',
  templateUrl: './usermanagement.component.html',
  styleUrls: ['./usermanagement.component.scss']
})
export class UsermanagementComponent implements OnInit {

  constructor(public http: HttpClient,public dialog: MatDialog,public snackBar: MatSnackBar,private tree: TreeService) { }
  private _transformer = (node, level: number) => {   //初始化tree
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      id: node.id,
      parentId: node.parentId,
      children: node.children
    };
  }
  treeControl = new FlatTreeControl<any>(node => node.level, node => node.expandable);
  treeFlattener = new MatTreeFlattener(this._transformer, node => node.level, node => node.expandable, node => node.children);
  treedataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  myControl = new FormControl();
  hasChild = (_: number, node: any) => node.expandable;

  ngOnInit() {
    this.initData()
    this.getAllMechanism()
  }
 

  displayedColumns: string[] = ['loginName', 'realName', 'mechanism','office','role','time', 'operation'];
  dataSource:any;   //所有用户
  allMechanism:any;   //所有机构

  userLogin:string; //搜索账号
  userName:string; //搜索名称
  userMechanism:string //搜索机构

  //分页
  @ViewChild(MatPaginator, {static: true})
  pageEvent: PageEvent;
  paginator: MatPaginator;
  length:any; //共多少条数据
  pageSize:any; //每页条数
  pageSizeOptions: number[] = [10] //设置每页条数
  pageNumber:number = 1; //第几页

  //分页切换
  chagePage (e) {
    this.pageNumber = e.pageIndex+1
    let data = {
      Name: this.userLogin ||'',
      RealName:this.userName ||'',
      OrganizationId:this.userMechanism ||'',
      PageNumber: String(this.pageNumber),
    }
    this.http.get('/api/Users',{params:data}).subscribe(
      (data:any[])=> {
        this.length = data['totalCount']
        this.pageSize = data['pageSize']
        this.dataSource = new MatTableDataSource<any>(data['items']);
      }
    )
  }

  isorganizationbox:any = false
  stopclose(e){
    e.stopPropagation();
  }
  js:any
  // jsId:any
  //点击辖区中队树，将选择的辖区中队添加到变量
  add(node) {
    this.isorganizationbox = false
    this.js = node.name
    // console.log(this.js)
    this.userMechanism = node.id
  }

  //打开辖区中队隐藏框
  openorganizationbox() {
    this.isorganizationbox = true
  }

  //关闭出现的组织机构div
  closediv(){
    this.isorganizationbox = false
  }
  
  //页面初始化 + 查询 + 重置
  initData () {
    let data = {
      Name: this.userLogin ||'',
      RealName:this.userName ||'',
      OrganizationId:this.userMechanism ||'',
      PageNumber: '1',
    }
    this.http.get('/api/Users',{params:data}).subscribe(
      (data:any[])=> {
        this.length = data['totalCount']
        this.pageSize = data['pageSize']
        this.pageEvent.pageIndex = 0
        this.dataSource = new MatTableDataSource<any>(data['items']);
      }
    )
  }

  //更新当前页数据
  getUsers () {
    let data = {
      Name: this.userLogin ||'',
      RealName:this.userName ||'',
      OrganizationId:this.userMechanism ||'',
      PageNumber: String(this.pageNumber),
    }
    this.http.get('/api/Users',{params:data}).subscribe(
      (data:any[])=> {
        this.length = data['totalCount']
        this.pageSize = data['pageSize']
        this.dataSource = new MatTableDataSource<any>(data['items']);
      }
    )

  }

  //清空自动搜索
  empty () {
    this.userLogin=''
    this.userName=''
    this.userMechanism=''
    this.js = ''
    this.isorganizationbox = false
    this.initData()
  }
  allorganizations:any//所有组织机构
  treedata:any//所有组织机构变成树
  newallorganizations:any

  //获取所有机构
  getAllMechanism() {
    this.http.get('/api/Organizations').subscribe(data=>{
      this.allorganizations = data
      this.treedata = this.tree.toTree(data);
      this.treedataSource.data = this.treedata
    })
  }

  //打开创建用户窗口
  open() {
    let dialogRef = this.dialog.open(CreateNewUser,{
      width: '850px',
    });
      dialogRef.afterClosed().subscribe(
        (data)=>{
          if (data) {this.getUsers()}
        }
      );
  }
  
  //打开分配角色窗口
  openuser (e) {
    let dialogRef = this.dialog.open(allRoles, 
      {      
       width:'600px',
        data: {e}
      });
    dialogRef.afterClosed().subscribe(
      (data)=>{
        if (data) {this.getUsers()}
      }
    ); 
  }

  //启用
  enable (e) {
    this.http.put(`/api/Users/${e.name}`,{
      name: e.name,
      realName:e.realName,
      post: e.post,
      phone: e.phone,
      enabled: true,
      creationTime:e.creationTime,
      organizationId:e.organizationId
    }).subscribe(
      data=> {
        this.getUsers()
      }
    )
  }

  //禁用
  prohibit (e) {
    this.http.put(`/api/Users/${e.name}`,{
      name: e.name,
      realName:e.realName,
      post: e.post,
      phone: e.phone,
      enabled: false,
      creationTime:e.creationTime,
      organizationId:e.organizationId
    }).subscribe(
      data=> {
        this.getUsers()
      }
    )
  }

  //编辑
  edit (e) {
    let allMechanism=this.allMechanism
    let dialogRef = this.dialog.open(EditNewUser, 
      {      
        data: {e,allMechanism}
      });

    dialogRef.afterClosed().subscribe(
      (data)=>{
        if (data) {this.getUsers()}
      }
    );   
  }

  //查看
  see (e) {
    this.http.get(`/api/Users/${e}`).subscribe(
      data=> {
        let dialogRef = this.dialog.open(seeInformation, 
          {      
           width:'600px',
            data
          });
        dialogRef.afterClosed().subscribe(); 
      }

    )
  }

  //重置密码
  reset (e) {
    this.http.put(`/api/Users/${e}/ResetPassword`,{}).subscribe(
      data=> {
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('重置密码成功!','确定',config);
      }
    )
  }
  
  //删除
  delete (e) {
    let isTrue = confirm('您确定要删除吗')
    if (isTrue) {
      this.http.delete(`/api/Users/${e}`).subscribe(
        data=>{
          this.getUsers()
        }
      )
    }
  }



}

//创建用户组件
@Component({
  selector: 'app-createUser',
  templateUrl: './createUser.html',
  styleUrls: ['./usermanagement.component.scss']
})
export class CreateNewUser {

  newdata = [];

  private _transformer = (node, level: number) => {
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      id: node.id,
      parentId: node.parentId,
      enabled:node.enabled,
      order:node.order,
    };
  }
  treeControl = new FlatTreeControl<any>(node => node.level, node => node.expandable);
  treeFlattener = new MatTreeFlattener(this._transformer, node => node.level, node => node.expandable, node => node.children);
  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  
  constructor(public http: HttpClient,public snackBar: MatSnackBar,
    public dialogRef: MatDialogRef<CreateNewUser>) {}

  organizationId:any='';   //所属机构id
  organizationName:any;  //所属机构name

  ngOnInit() {
    this.tree()
  }

  //渲染Tree
  tree () {
    this.newdata=[]
    this.http.get('/api/Organizations').subscribe(
      (data:any)=>{
        function getparentNode(parentId){
          return data.find((item)=>{
            return item.id == parentId
          })
        }
        data.forEach(item => {
          var parentNode =  getparentNode(item.parentId);
          if(parentNode){
            if(!parentNode.children){
              parentNode.children = []
            }
            parentNode.children.push(item)
          }else{
            if(!item.parentId){
              this.newdata.push(item)
            }
          }
        });
        this.dataSource.data = this.newdata;
      }
    )

  }
  hasChild = (_: number, node: any) => node.expandable;

  //选择机构
  add (e) {
    this.organizationId = e.id
    this.organizationName = e.name
    this.selectedOffices = {}
    this.getOffices(e)
  }
  selectedOrg:any//当前选择的组织机构
  OfficesList:any //获得的下级列表
  //获得点击组织机构的下级机关单位
  getOffices(node){
    // console.log(node)
    this.selectedOrg = node
    this.http.get(`/api/OrganizationsOffices/${node.id}`).subscribe(data => {
      // console.log("获取成功",data)
      this.OfficesList = data
    })
  }
  //选择机关
  selectedOffices:any = {}
  selectOffice(item){
    this.selectedOffices = item
    // console.log(item)
  }
  errmsg:any; //捕获错误信息
  //新增用户
  onSubmit(e) {
    let time =new Date()
    if(this.organizationId) {
      this.http.post('/api/Users',{
        name:e.name,
        realName:e.user,
        post:e.post,
        identityCard:e.identityCard,
        phone:e.phone,
        enabled:true,
        creationTime:time,
        organizationId:this.organizationId,
        organizationOfficeId:this.selectedOffices.id
      }).subscribe(data=> { 
        this.dialogRef.close('success')},
        error=>{this.errmsg = error}
      )
    }else{
      const config = new MatSnackBarConfig();
      config.verticalPosition = 'top';
      config.duration = 3000
      this.snackBar.open('请选择机构','确定',config);
    }
  }



}



//编辑用户组件
@Component({
  selector: 'app-editUser',
  templateUrl: './editUser.html',
  styleUrls: ['./usermanagement.component.scss']
})
export class EditNewUser {
  constructor(public http: HttpClient,public snackBar: MatSnackBar,
    public dialogRef: MatDialogRef<EditNewUser>,@Inject(MAT_DIALOG_DATA) public data) {}

  ngOnInit() {
    this.getTreeData()
    this.name=this.data.e.name
    this.realName=this.data.e.realName
    this.identityCard = this.data.e.identityCard
    this.post=this.data.e.post,
    this.phone=this.data.e.phone,
    this.affiliate=this.data.e.organizationId
    this.affiliateName=this.data.e.organizationName
    this.selectedOffices.officeName = this.data.e.organizationOfficeName
    this.selectedOffices.id = this.data.e.organizationOfficeId
    // console.log(this.data)
  }

  private _transformer = (node, level: number) => {
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      id: node.id,
      parentId: node.parentId,
      enabled:node.enabled,
      order:node.order,
    };
  }
  treeControl = new FlatTreeControl<any>(node => node.level, node => node.expandable);
  treeFlattener = new MatTreeFlattener(this._transformer, node => node.level, node => node.expandable, node => node.children);
  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  hasChild = (_: number, node: any) => node.expandable;

  name:any; //登录账号
  realName:any; //真实姓名
  post:any; //职务
  phone:any; //联系电话
  affiliate:any; //所属机构id
  affiliateName:any; //所属机构name

  //获取tree数据
  getTreeData () {
    this.http.get('/api/Organizations').subscribe((data:any)=>{
      let treeData = []
      data.forEach(element => {
        element.children = []
        data.forEach(item => { if (item.parentId==element.id) {element.children.push(item)} });
        if(!element.parentId) {treeData.push(element)}
      }); //forEach
      this.dataSource.data = treeData

    })
  }

  //选择组织机构
  add (e) {
    this.affiliate = e.id
    this.affiliateName = e.name
    this.selectedOffices = {}
    this.getOffices(e)
  }
    selectedOrg:any//当前选择的组织机构
    OfficesList:any //获得的下级列表
    //获得点击组织机构的下级机关单位
    getOffices(node){
      // console.log(node)
      this.selectedOrg = node
      this.http.get(`/api/OrganizationsOffices/${node.id}`).subscribe(data => {
        // console.log("获取成功",data)
        this.OfficesList = data
      })
    }
    //选择机关
    selectedOffices:any = {}
    selectOffice(item){
      this.selectedOffices = item
      // console.log(item)
    }
  //编辑
  identityCard:any//身份证
  onSubmit (e) {
    this.http.put(`/api/Users/${this.data.e.name}`,{
      name:this.data.e.name,
      realName:this.realName,
      post:this.post,
      identityCard:this.identityCard,
      phone:this.phone,
      roleNames:this.data.e.roleNames,
      enabled: this.data.e.enabled,
      creationTime:this.data.e.creationTime,
      organizationId:this.affiliate,
      organizationOfficeId:this.selectedOffices.id
    }).subscribe(data=>{
      this.dialogRef.close('success')
    })

  }



}



//分配角色组件
@Component({
  selector: 'app-allRoles',
  templateUrl: './allRoles.html',
  styleUrls: ['./usermanagement.component.scss']
})
export class allRoles {

  constructor(public http: HttpClient,public snackBar: MatSnackBar,
    public dialogRef: MatDialogRef<allRoles>,
    @Inject(MAT_DIALOG_DATA) public data) {}

    ngOnInit() {
      this.getUsers()
    }

    allRoles:any; //所有角色
    checked:any=[]; //选中的角色

    //获取所有角色
    getUsers () {
      this.http.get('/api/Roles').subscribe( data=>{
        this.allRoles=data
        this.allRoles.forEach(element => { //所有角色
          element.checked = false
          this.data.e.roleNames.forEach(elements=> { //当前账号已有角色
            if (element.name == elements) {
              element.checked = true
              this.checked.push(element)} });
        });

      })
    }

    //check框change事件
    fill (e) {
      e.checked = !e.checked
      if (e.checked) {
        this.checked.push(e)
      } else {
        this.checked.splice(this.checked.findIndex(item => item==e), 1)
      }
    }

    //分配角色
    onSubmit () {
      if (this.checked.length) {
        let data:any=[]
        this.checked.forEach(item=>{
          data.push(item.id)
        })
       this.http.post(`/api/Users/${this.data.e.name}/Roles`,data).subscribe( data=> {
          this.dialogRef.close('success')
        })
      } else {
        const config = new MatSnackBarConfig();
        config.verticalPosition = 'top';
        config.duration = 3000
        this.snackBar.open('请选择分配角色','确定',config);
      }
    }



}


//查看用户组件
@Component({
  selector: 'app-seeInformation',
  templateUrl: './seeInformation.html',
  styleUrls: ['./usermanagement.component.scss']
})
export class seeInformation {
  constructor(public http: HttpClient,
    public dialogRef: MatDialogRef<seeInformation>,
    @Inject(MAT_DIALOG_DATA) public data) {}

    ngOnInit() {
    }

}
